@@ -1,10 +1,9 @@
-#########1 Main Package       3#########4#########5#########6#########7#########8#########9
 package MooseX::ShortCut::BuildInstance;
 our $AUTHORITY = 'cpan:JANDREW';
 # ABSTRACT: A shortcut to build Moose instances
 
-use version 0.77; our $VERSION = version->declare('v1.34.8');
-###LogSD	warn "You uncovered internal logging statements for Spreadsheet::XLSX::Reader::LibXML-$VERSION";
+use version 0.77; our $VERSION = version->declare('v1.38.2');
+###LogSD	warn "You uncovered internal logging statements for MooseX::ShortCut::BuildInstance-$VERSION";
 use 5.010;
 use Moose 2.1213;
 use Moose::Meta::Class;
@@ -23,12 +22,10 @@ Moose::Exporter->setup_import_methods(
 use Data::Dumper;
 use Clone 'clone';
 use lib	'../../../lib',;
-use MooseX::ShortCut::BuildInstance::Types 1.034 qw(
+use MooseX::ShortCut::BuildInstance::Types 1.036 qw(
 		BuildClassDict
 	);
-###LogSD warn "You uncovered internal logging statements for MooseX::ShortCut::BuildInstance!";
 ###LogSD use Log::Shiras::Telephone;
-#~ ###LogSD use MooseX::ShortCut::BuildInstance::UnhideDebug;
 
 #########1 Package Variables  3#########4#########5#########6#########7#########8#########9
 
@@ -94,6 +91,17 @@ sub build_class{
 		cluck( join( "\n", @warn_list ) );
 		###LogSD	$phone->talk( level => 'warn', message =>[
 		###LogSD		'The old class definitions will be overwritten with args:',  $class_args ] );
+	}else{
+		my $package_key = $class_args->{package};
+		$package_key =~ s/::/\//g;
+		$package_key .= '.pm';
+		if( exists $INC{$package_key} ){
+			return $class_args->{package} if $re_use_classes;# Don't rebuild if you are re-using
+			cluck "Overwriting a pre-built and loaded class: " . $class_args->{package} ;
+			###LogSD	$phone->talk( level => 'warn', message =>[
+			###LogSD		"unmutablizing the class: $class_args->{package}" ] );
+			$class_args->{package}->meta->make_mutable;
+		}
 	}
 	my $want_array = ( caller(0) )[5];
 	###LogSD	$phone->talk( level => 'trace', message =>[
@@ -244,12 +252,12 @@ MooseX::ShortCut::BuildInstance - A shortcut to build Moose instances
 	<img alt="Build Status" src="https://travis-ci.org/jandrew/MooseX-ShortCut-BuildInstance.png?branch=master" alt='Travis Build'/>
 </a>
 
-<a href='https://coveralls.io/r/jandrew/MooseX-ShortCut-BuildInstance?branch=master'>
-	<img src='https://coveralls.io/repos/jandrew/MooseX-ShortCut-BuildInstance/badge.svg?branch=master' alt='Coverage Status' />
+<a href='https://coveralls.io/github/jandrew/MooseX-ShortCut-BuildInstance?branch=master'>
+	<img src='https://coveralls.io/repos/github/jandrew/MooseX-ShortCut-BuildInstance/badge.svg?branch=master' alt='Coverage Status' />
 </a>
 
-<a>
-	<img src="https://img.shields.io/badge/this version-1.34.8-brightgreen.svg" alt="this version">
+<a href='https://github.com/jandrew/MooseX-ShortCut-BuildInstance'>
+	<img src="https://img.shields.io/github/tag/jandrew/MooseX-ShortCut-BuildInstance.svg?label=github version" alt="github version"/>
 </a>
 
 <a href="https://metacpan.org/pod/MooseX::ShortCut::BuildInstance">
@@ -264,21 +272,28 @@ MooseX::ShortCut::BuildInstance - A shortcut to build Moose instances
 
 =head1 SYNOPSIS
     
-	#!perl
+	#!/user/bin/env perl
 	package Mineral;
 	use Moose;
+	use Types::Standard qw( Enum );
 
-	has 'type' =>( is => 'ro' );
+	has 'type' =>( 
+			isa => Enum[qw( Quartz Diamond Basalt Granite )],
+			is => 'ro' 
+		);
 
 	package Identity;
 	use Moose::Role;
 
 	has 'name' =>( is => 'ro' );
 
-	use MooseX::ShortCut::BuildInstance;
+	use lib '../../../lib';
+	use MooseX::ShortCut::BuildInstance qw( should_re_use_classes build_instance );
+	should_re_use_classes( 1 );# To reuse build_instance
 	use Test::More;
 	use Test::Moose;
 
+	# First build of instance
 	my 	$paco = build_instance(
 			package => 'Pet::Rock',
 			superclasses =>['Mineral'],
@@ -290,27 +305,49 @@ MooseX::ShortCut::BuildInstance - A shortcut to build Moose instances
 	does_ok( $paco, 'Identity', 'Check that the ' . $paco->meta->name . ' has an -Identity-' );
 	print'My ' . $paco->meta->name . ' made from -' . $paco->type . '- (a ' .
 	( join ', ', $paco->meta->superclasses ) . ') is called -' . $paco->name . "-\n";
+
+	# Next instance (If you don't want to call build_instance again)
+	my $Fransisco = Pet::Rock->new(
+		type => 'Diamond',
+		name => 'Fransisco',
+	);
+	does_ok( $Fransisco, 'Identity', 'Check that the ' . $Fransisco->meta->name . ' has an -Identity-' );
+	print'My ' . $Fransisco->meta->name . ' made from -' . $Fransisco->type . '- (a ' .
+	( join ', ', $Fransisco->meta->superclasses ) . ') is called -' . $Fransisco->name . "-\n";
+
+	# Another instance (reusing build_instance)
+	my $Gonzalo = build_instance(
+			package => 'Pet::Rock',
+			superclasses =>['Mineral'],
+			roles =>['Identity'],
+			type => 'Granite',
+			name => 'Gonzalo',
+		);
+	does_ok( $Gonzalo, 'Identity', 'Check that the ' . $Gonzalo->meta->name . ' has an -Identity-' );
+	print'My ' . $Gonzalo->meta->name . ' made from -' . $Gonzalo->type . '- (a ' .
+	( join ', ', $Gonzalo->meta->superclasses ) . ') is called -' . $Gonzalo->name . "-\n";
 	done_testing();
     
     ##############################################################################
     #     Output of SYNOPSIS
     # 01:ok 1 - Check that the Pet::Rock has an -Identity-
     # 02:My Pet::Rock made from -Quartz- (a Mineral) is called -Paco-
-    # 03:1..1
+    # 01:ok 1 - Check that the Pet::Rock has an -Identity-
+    # 02:My Pet::Rock made from -Diamond- (a Mineral) is called -Fransisco-
+    # 01:ok 1 - Check that the Pet::Rock has an -Identity-
+    # 02:My Pet::Rock made from -Granite- (a Mineral) is called -Gonzalo-
+    # 03:1..3
     ##############################################################################
-
     
 =head1 DESCRIPTION
 
 This module is a shortcut to custom build L<Moose> class instances on the fly.  
 The goal is to compose unique instances of Moose classes on the fly using a single 
 function call with information describing required attributes, methods, inherited 
-classes, and roles as well as any instance settings to apply in a 
-L<DCI|https://en.wikipedia.org/wiki/Data,_Context,_and_Interaction> fashion.  
-This package will check for and fill in any missing pieces as needed so that your 
-call can either be complex or very simple.  The goal is to provide configurable 
-instance building without stringing together a series of Class-E<gt>method( %args ) 
-calls.
+classes, and roles as well as any desired instance settings.  This package will 
+check for and fill in any missing pieces as needed so that your call can either be 
+complex or very simple.  The goal is to provide configurable instance building 
+without stringing together a series of Class-E<gt>method( %args ) calls.
 
 The package can also be used as a class factory with the L<should_re_use_classes
 |/$MooseX::ShortCut::BuildInstance::re_use_classes> method.
@@ -325,7 +362,8 @@ on the fly while building an instance and then recompose a new class (package) w
 the same name but different functionality (different attributes, methods, inherited 
 classes or roles) while composing a new instance on the fly then all calls 
 to the old instance will use the new class functionality for execution. (Usually 
-causing hard to troubleshoot failures)
+causing hard to troubleshoot failures).  This is also true if you re-build a 
+prebuilt class name inititally installed with 'use'.
 
 MooseX::ShortCut::BuildInstance will warn if you overwrite named classes (packages)
 built on top of another class (package) also built by MooseX::ShortCut::BuildInstance.  
@@ -339,6 +377,9 @@ against the resulting class name over and over again.  Another alternative is to
 leave the 'package' argument out of 'build_instance' and let this class create a 
 unique by-instance anonymous class/package name.
 
+MooseX::ShortCut::BuildInstance will also warn (but not stop you) if you try to 
+overwrite a pre-loaded package initially installed with 'use' or 'require'.
+
 The Types module in this package uses L<Type::Tiny> which can, in the 
 background, use L<Type::Tiny::XS>.  While in general this is a good thing you will 
 need to make sure that Type::Tiny::XS is version 0.010 or newer since the older 
@@ -526,7 +567,8 @@ with the same name.  If you wish to avoid changing old built behaviour at the ri
 of not installing new behaviour then set this variable to true.  I<No warning will 
 be provided if new requested class behaviour is discarded.> The class reuse behaviour 
 can be changed with the exported method L<should_re_use_classes
-|/should_re_use_classes( $bool )>.
+|/should_re_use_classes( $bool )>.  This does not apply to pre-loaded classes.  
+For pre-loaded classes this package will cluck and then overwrite every time.
 
 =over
 
@@ -616,7 +658,7 @@ it and/or modify it under the same terms as Perl itself.
 The full text of the license can be found in the
 LICENSE file included with this module.
 
-This software is copyrighted (c) 2012, 2015 by Jed Lund
+This software is copyrighted (c) 2012, 2016 by Jed Lund
 
 =head1 Dependencies
 
@@ -661,10 +703,7 @@ L<Log::Shiras|https://github.com/jandrew/Log-Shiras>
 
 =over
 
-This package does not use Log::Shiras functionality by default.  The functionality is 
-only turned on if the variable $ENV{log_shiras_filter_on} is set to true.  Otherwise 
-all the Log::Shiras code is hidden as comments.  See 
-L<MooseX::ShortCut::BuildInstance::UnhideDebug> for more information.
+All lines in this package that use Log::Shiras are commented out
 
 =back
 
